/************************************************************************************/
/*	myActionProc																	*/
/*	Called when there is a mouse click on a control part.							*/
/*																					*/
/*	Note:	although the physical location of destRect doesn't change, scrolling	*/
/*			and autoscroll change the value of destRect.top							*/
/*																					*/
/*	Thanks to the following for contributions to fix an error which was preventing	*/
/*	this routine from running in 32-bit mode on Mac-IIci and above:					*/
/*																					*/
/*	Jim Ulrey and 							-	Identified the error, and verified	*/
/*	Paul Bennell (paul@chiron.com.au)			that it only occured in 32-bit		*/
/*												mode.  Isolated the problem to the	*/
/*												call to TrackAction when the		*/
/*												mouseDown event was not in the		*/
/*												thumbwheel part of the control.		*/
/*																					*/
/*	Marcelino Bernardo (mBernar@erenj.com)	-	Found the problem in the incorrect	*/
/*												definition of myActionProc as		*/
/*												'pascal Boolean' instead of			*/
/*												'pascal void'.						*/
/************************************************************************************/

#include "MyHeaders.h"

pascal void myActionProc(ControlHandle theControl, short partCode)		/* 5.1.2	*/
{
	WindowPtr		theWindow;
	TEHandle		theTE;
	short				vView;						/* height of viewRect rounded		*/
	short				vViewP;						/* height of viewRect not rounded	*/
	short				vLine;						/* height of one line of text		*/
	short				vTotal;						/* total height of text				*/
	short				vAmount;					/* total amount that can be scroled	*/
	short				vScrlAmt;					/* scroll amount					*/
	div_t			r;							/* work area for division			*/
	short				tAdjust;					/* amount to adjust thumb scrolling	*/
	short				teHSub;						/* subscript of active TE item		*/

	teHSub = windTbl[windSub].windTEHSub;					/* set the subscript	*/
	
	theWindow = ((WindowPtr) (**theControl).contrlOwner);		/* ptr to window	*/
	WhichWindow (theWindow, &windSub);							/* subscr to table	*/
	theTE = windTbl[windSub].windTEH[teHSub];					/* handle to TE rec	*/

	vLine = (**theTE).lineHeight;								/* height of 1 line	*/
	vViewP = (**theTE).viewRect.bottom -(**theTE).viewRect.top;	/* height of 1 page	*/
	vView = (vViewP / vLine) * vLine;							/* rounded to lines	*/
	vTotal = (**theTE).nLines * (**theTE).lineHeight;			/* height whole txt	*/
	if ((**theTE).teLength > 0 									/* if ends in c/r	*/
		&& (*((**theTE).hText))[(**theTE).teLength-1]=='\r')	/*    adjust for it	*/
			vTotal += (**theTE).lineHeight;

	switch (partCode)
		{
		case (inUpButton):
			vScrlAmt = ((**theTE).viewRect.top > (**theTE).destRect.top + vLine)
					? vLine
					: (**theTE).viewRect.top - (**theTE).destRect.top;
			if (vScrlAmt < 0)					/* if calculation has wrong sign	*/
				vScrlAmt = 0;					/* don't scroll						*/
		break;
				
		case (inPageUp):
			vScrlAmt = ((**theTE).viewRect.top > (**theTE).destRect.top + vView)
					? vView
					: (**theTE).viewRect.top - (**theTE).destRect.top;
			if (vScrlAmt < 0)					/* if calculation has wrong sign	*/
				vScrlAmt = 0;					/* don't scroll						*/
		break;
		
		case (inPageDown):
			vScrlAmt = ((**theTE).viewRect.bottom 
						< (**theTE).destRect.top + vTotal - vView) 
					? -vView 
					: -((**theTE).destRect.top + vTotal - (**theTE).viewRect.bottom);
			if (vScrlAmt > 0)					/* if calculation has wrong sign	*/
				vScrlAmt = 0;					/* don't scroll						*/
		break;
				
		case (inDownButton):
			vScrlAmt = ((**theTE).viewRect.bottom 
						< (**theTE).destRect.top + vTotal - vLine) 
					? -vLine 
					: -((**theTE).destRect.top + vTotal - (**theTE).viewRect.bottom);
			if (vScrlAmt > 0)					/* if calculation has wrong sign	*/
				vScrlAmt = 0;					/* don't scroll						*/
		break;
		
		case (inThumb):
												/* current value of control			*/
			j = GetCtlValue (theControl);		/*   after moving thumb				*/
			
												/* difference in control value		*/
			vScrlAmt = k;						/*   due to moving thumb			*/
			
												/* how much more or less than whole	*/
			r = div (vScrlAmt, vLine);			/*   number of lines in the scroll?	*/
			
												/* compute adjustment to the scroll	*/
												/*   amount and the control value	*/
												/*   to result in only scrolling a	*/
												/*   whole number of lines			*/
												
			if (abs(r.rem) < vLine/2)				/* if we should round down		*/
				tAdjust = -r.rem;					/*   use this					*/
			else									/* else							*/
				{									/* if we should round up		*/
				if (r.rem > 0)						/*   if we are scrolling up		*/
					tAdjust = -r.rem + vLine;		/*		use this				*/
				else								/*   else if scrolling down		*/
					tAdjust = -r.rem - vLine;		/*      use this				*/
				}
					
			vScrlAmt += tAdjust;				/* adjust scroll amt (scroll later)	*/
			
			if (tAdjust != 0)					/* adjust control amount and redraw	*/
				SetCtlValue (theControl, (j - tAdjust));
		break;
		
		case (0):
			vScrlAmt = 0;
												/* If the total vertical length of	*/
												/* the text now is less than the	*/
												/* height of the viewRect, prepare	*/
												/* to scroll to the top of viewRect	*/
			if ((vViewP >= vTotal)
					&& ((**theTE).viewRect.top != (**theTE).destRect.top))
				vScrlAmt = (**theTE).viewRect.top - (**theTE).destRect.top;	
		break;
		
		default:
			return;											/* 5.1.2				*/
		}
		
	TEScroll (0, vScrlAmt, theTE);							/* do the scroll		*/			

	vAmount = (vTotal > vViewP) ? vTotal-vViewP : 0;		/* ..max scrollable amt	*/
	if (vAmount == 0)										/* ..if view > text		*/
		(**theTE).destRect.top = (**theTE).viewRect.top;	/* ..  fit all in view	*/
	
	if (GetCtlMax (theControl) != vAmount)					/* set max to..			*/
		SetCtlMax(theControl, vAmount);						/* .. max scrollable	*/
		
	if (partCode != inThumb)								/* reset and redraw		*/
		SetCtlValue (theControl, ((**theTE).viewRect.top) - (**theTE).destRect.top);
	
	return; 												/* 5.1.2				*/
}
