/*****************************************************************************************************
*                                                                                                    *
* Finder ProgressBar.c - Copyright 1993 Chris Larson (cklarson@engr.ucdavis.edu) All rights reserved *
*                                                                                                    *
* Source file of a CDEF which mimics the progress bar used in the Finder. This source file and its   *
* compiled derivatives may be freely used within any freeware/shareware/postcardware/beerware/ as   *
* long as you mention my name in your credits. Neither this source nor its compiled derivatives are  *
* in the public domain and may not be use in any form in public domain software. Neither this source *
* nor its compiled derivatives may be used in any form in a commercial product without the expressed,*
* written consent of the author (me).                                                                *
*                                                                                                    *
* Version 1.0 -- Initial Release.                                                                    *
*                                                                                                    *
*****************************************************************************************************/

#include "Finder ProgressBar.h"

//----------------------------------------------------------------------------------------------------
//
// main -- Handle everything but draw messages.
//
//----------------------------------------------------------------------------------------------------

pascal long main (short varCode, ControlHandle theControlHandle, short message, long param)
{
	SignedByte	controlRecState;	// Holds the state of the control records handle.
	long		returnValue;		// Holds the value returned by the CDEF.
	ControlPtr	theControl;			// Holds a pointer to the control record.
	
	// ----------
	// Since I use some static local variables within the draw routine, a4 must be set up
	// (read: made to point to the top of the CDEF) to access global and static variables.
	// This routine simply saves the value of a4 and points a4 to the top of the CDEF
	// (this address is put into a0 by the header so I simply read it from there). See the
	// header file for details.
	// ----------
	
	MySetUpA4();
	
	// ----------
	// Lock down the control record.
	// ----------
	
	controlRecState = HGetState((Handle)theControlHandle);
	HLock((Handle)theControlHandle);
	
	// ----------
	// Initialize the return value and the control pointer (since the record is locked it is safe
	// to use a pointer). The return value is initialized to 0 since most CDEF messages want 0
	// returned.
	// ----------
	
	returnValue = 0;
	theControl = *theControlHandle;
	
	switch (message)
		{
		
		// ----------
		// Ignore draw messages if the control is not visible.
		// ----------
		
		case drawCntl:
			if (theControl->contrlVis)
				DrawProgressBar(theControlHandle);
			break;
		
		// ----------
		// Test the control: return 1 if the give point is within the control's rectangle
		// and the control is not dimmed, 0 otherwise (the whole bar is part #1).
		// ----------
		
		case testCntl:
			if (theControl->contrlHilite != 0xFF)
				returnValue = PtInRect(*(Point*)(&param),&(theControl->contrlRect));
			break;
		
		// ----------
		// For the 24-bit mode region calculation message, the high bit of the given region handle
		// must be cleared. Note that I dont test this bit to determine which region to calculate.
		// Since this control has no indicator (part with a part code >= 129) I will never receive
		// a message asking for indicator region calculation.
		// ----------
		
		case calcCRgns:
			param &= 0x7FFFFFFF;
		
		// ----------
		// Calculate the controls region.
		// ----------
		
		case calcCntlRgn:
			RectRgn((RgnHandle)param,&(theControl->contrlRect));
			break;
		
		// ----------
		// All other messages do nothing and return 0.
		// ----------
		
		default:
			break;
		}
	
	// ----------
	// Restore the state of the control records handle.
	// ----------
	
	HSetState ((Handle)theControlHandle, controlRecState);
	
	// ----------
	// Restore the value of a4 from its saved location.
	// ----------
	
	MyRestoreA4();
	
	// ----------
	// Were outa here.
	// ----------
	
	return (returnValue);
}

//----------------------------------------------------------------------------------------------------
//
// DrawProgressBar -- Draw the control according to the values within the control record.
//
//----------------------------------------------------------------------------------------------------

void DrawProgressBar (ControlHandle theControlHandle)
{
	PenState		oldPen;
	RGBColor		oldFore, oldBack;
	static RGBColor	blackColor = cBlack, whiteColor = cWhite, blueColor = cBlue, grayColor = cGray;
	RgnHandle		oldClip,currentClip;
	ControlPtr		theControl = *theControlHandle;
	Rect			theBox = theControl->contrlRect;
	GrafPtr			savePort, controlPort = theControl->contrlOwner;
	
	// ----------
	// Set the port to the controls port. Very probably not needed (since Apples CDEFs do not
	// do this) but just to be extra safe
	// ----------
	
	GetPort (&savePort);
	SetPort (controlPort);
	
	// ----------
	// Save the current ports clip region and set it to the intersection of the clip region with
	// the controls rectangle.
	// ----------
	
	oldClip = NewRgn();
	GetClip(oldClip);
	
	ClipRect(&(theControl->contrlRect));
	currentClip = theControl->contrlOwner->clipRgn;
	SectRgn(currentClip,oldClip,currentClip);
	
	// ----------
	// Save and reset the pen state
	// ----------
	
	GetPenState(&oldPen);
	PenNormal();
	
	// ----------
	// If we have color QD, save the foreground and background colors and set the foreground color
	// to black, background color to white.
	// ----------
	
	if ( (ROM85 & kNoColorQD) == 0 )
		{
		GetForeColor(&oldFore);
		GetBackColor(&oldBack);
		
		RGBForeColor(&blackColor);
		RGBBackColor(&whiteColor);
		}
	
	// ----------
	// Draw the progress bar frame and set up the rectangle for drawing the bar.
	// ----------
	
	FrameRect(&theBox);
	InsetRect(&theBox,1,1);
	
	// ----------
	// CalcBarBoundry is an inline function that returns the value described by the equation
	// theBox.left + ( controlValue * (theBox.right - theBox.left) )/(controlMax - controlMin).
	// This is implemented as an inline function to avoid using the glue for 32-bit multiplication
	// and division that is necessary on the 68000 (the glue is not really needed for this
	// calculation and it more than doubles the size of the CDEF when included, so I rolled my
	// own).
	// ----------
	
	theBox.right = CalcBarBoundry(&theBox,theControl);
	
	// ----------
	// If we have color, set the foreground color to gray, leave the background white so that
	// the gray will map to black on a 1-bit monitor. Note that for non-color QD machines, we want
	// the bar to be black. Since the PenNormal() call set the pen pattern to black, we need do
	// nothing. Then draw the bar.
	// ----------
	
	if ( (ROM85 & kNoColorQD) == 0 )
		RGBForeColor(&grayColor);

	PaintRect(&theBox);
	
	 // ----------
	 // Set up the rectangle to describe the space from the end of the bar to the edge of the
	 // inset rectangle.
	 // ----------
	 
	theBox.left = theBox.right;
	theBox.right = (theControl->contrlRect.right) - 1;

	// ----------
	// If we have color, set the foreground to blue and the background to black (so that the blue
	// will map to white on a 1-bit monitor). If no color, we want to erase the empty space, so
	// set the pen to erase. Then paint the rectangle.
	// ----------

	if ( (ROM85 & kNoColorQD) == 0 )
		{
		RGBForeColor(&blueColor);
		RGBBackColor(&blackColor);
		}
	else
		PenMode(patBic);
	
	PaintRect(&theBox);
	
	// ----------
	// Restore all the stuff we munged: colors, pen state, clip region, and current port.
	// ----------
	
	if ( (ROM85 & kNoColorQD) == 0 )
		{
		RGBForeColor(&oldFore);
		RGBBackColor(&oldBack);
		}
	
	SetPenState(&oldPen);
	
	SetClip(oldClip);
	DisposeRgn(oldClip);
	
	SetPort(savePort);
}