/*
**	List dialog prototype
**	(C) Bernie Wieser 1990 (BSWieser@UNCAMULT.BITNET)
**
**	Purpose 
**		To use the lists from within a dialog box.  A user item rect
**	is defined in a dialog template to be used by the list manager for
**	a text only, one column list.
**		If you use it or have questions/comments, please send me mail.
**	This is only an example...
**
**	Problems
**		Doesn't the list manager provide routines for cursor keys?
**
*/

#include "list&dialog.h"
#include <ListMgr.h>
#include <DialogMgr.h>

/*
**	The buttons from the dialog defined.
*/
#define NULL	0
#define OPEN	1
#define QUIT	2
#define SCROLL	3

/*
**	Globals
**		lh - the list record handle
**		stuff - an array of test strings
*/
ListHandle lh;

char *stuff[] = {
	"\phello",
	"\pdolly",
	"\phow've",
	"\pyou",
	"\pbeen",
	"\pdolly",
	"\pyou",
	"\pknow",
	"\pmy",
	"\plife",
	"\phas",
	"\pbeen",
	"\preal",
	"\pdull",
	"\psince",
	"\pyou've",
	"\pbeen",
	"\pgone!"};

/*
**	Check to see if ANY cell is active
*/
Boolean cell_active()
{
		long nil = 0L;
		return(LGetSelect(TRUE, &nil, lh));
}

/*
**	Draw the dialog with list
*/
pascal void draw_list(my_dlog, item_no)
DialogPtr my_dlog;
short item_no;
{
	int item_type;
	Handle item;
	Rect box;
	
	SetPort(my_dlog);
	
	/* draw the list */
	GetDItem(my_dlog, item_no, &item_type, &item, &box);
	LUpdate((**lh).port->visRgn, lh);
	InsetRect(&box, -1, -1);
	FrameRect(&box);
}

/*
**	The event filter for the list dialog
**	Unfortunately, I have to call this routine here to maintain the
**	list because it handles events sent to the list.
*/
pascal Boolean filter_list(my_dlog, the_event, item_hit)
DialogPtr my_dlog;
EventRecord *the_event;
int *item_hit;
{
	Point where;		/* where click occurred */
	int tc,				/* the control part		*/
		key;			/* the key depressed	*/
	ControlHandle ch;	/* a control handle		*/
	
	Boolean status = FALSE;
	
	/* handle events, return FALSE if modal dlog should handle it	*/
	/* ONLY HANDLE LIST EVENTS!  LET MODAL DLOG DO THE REST!		*/
	SetPort(my_dlog);
	
	/* handle the cursor keys etc.  Isn't there an easier way?		*/
	/* fake mouse click by subracting box width from current clikloc*/
	if ((the_event->what == keyDown) || (the_event->what == autoKey)) {
		ObscureCursor();
		key = the_event->message & charCodeMask;
		where.h = (**lh).clikLoc.h;
		LAutoScroll(lh);   /* has wierd effects... */
		switch(key) {
			/* enter/return key */
			case 0x03:
			case 0x0d:
				*item_hit = 1;
				return(TRUE);
				break;
			/* up arrow */
			case 0x1e:
				/* check scroll up */
				where.v = (**lh).clikLoc.v - (**lh).cellSize.v;
				if(!PtInRect(where, &(**lh).rView)) {
					/* can I scroll up? */
					LScroll(0, -1, lh);
					where.v = (**lh).clikLoc.v;
					}
				status = TRUE;
				break;
			/* down arrow */
			case 0x1f:
				where.v = (**lh).clikLoc.v + (**lh).cellSize.v;
				if(!PtInRect(where, &(**lh).rView)) {
					/* can I scroll down? */
					LScroll(0, 1, lh);
					where.v = (**lh).clikLoc.v;
					}
				status = TRUE;
				break;
			/* un-processed key */
			default:
				SysBeep(5);
				break;
			}
		if(status) { /* if movement */
			LClick(where, the_event->modifiers, lh);
			*item_hit = NULL;
			}
		}

	if (the_event->what == mouseDown) {
		where = the_event->where;
		GlobalToLocal(&where);
		
		/* find the control handle for scroll bar click etc. */
		tc = FindControl(where, my_dlog, &ch);
		
		/* if it is a mouse down in the list or scroll bar (which is	*/
		/* disjoint from the user item rect!!!) then handle it.			*/
		if(PtInRect(where, &(**lh).rView) || (ch == (**lh).vScroll)) {
			/* see if double click - if so, true */
			if(LClick(where, the_event->modifiers, lh)) {
				/* reset response to item 1 */
				*item_hit = OPEN;
				}
			else {
				/* clear the response, single click or in scroll bars */
				*item_hit = NULL;
				}
			status = TRUE;
			}
		}
	return(status);
}

/*
**	Do a dialog
*/
do_dialog(window)
WindowPtr window;
{
	DialogPtr my_dlog = GetNewDialog(4000, NIL, INFRONT);
	int x, item_type, item_hit = 0;
	Handle item;
	Cell cell;
	Rect box, list;
	Boolean done = FALSE;
	
	SetPort(my_dlog);
	
	GetDItem(my_dlog, SCROLL, &item_type, &item, &box);
	/* insert my draw procedure for user item SCROLL */
	SetDItem(my_dlog, SCROLL, item_type, draw_list, &box);
	
	/* define list bounds and make a new list */
	SetRect(&list, 0, 0, 1, 18);
	lh = LNew(&box, &list, NIL, NIL, my_dlog, FALSE, FALSE, FALSE, TRUE);
	
	SetPt(&cell, 0, 0); /* start list at (0,0) */
	LSetSelect(TRUE, cell, lh); /* and select it */
	/* must set click lock for keyboard to work */
	SetPt(&(**lh).clikLoc, box.left+1, box.top+1);
	SetPt(&(**lh).lastClick, 0, 0);
	
	/* build the list */
	for(x=0; x<18; x++) {
		cell.v = x;
		LSetCell(stuff[x]+1, *stuff[x], cell, lh);
		}

	LDoDraw(TRUE, lh);
	ShowWindow(my_dlog);
	
	/* handle the dialog */
	while(!done) {
	
		/* first, handle any controls which shouldn't be accessable	*/
		/* i.e. if no cell is selected, then don't use OPEN			*/
		if(cell_active()) {
			GetDItem(my_dlog, OPEN, &item_type, &item, &box);
			HiliteControl(item, 0);
			SetDItem(my_dlog, OPEN, item_type, item, &box);
			/* make it default */
			InsetRect(&box,-4,-4);
			PenSize(3,3);
			FrameRoundRect(&box, 16, 16);
			}
		else {
			GetDItem(my_dlog, OPEN, &item_type, &item, &box);
			HiliteControl(item, 255);
			SetDItem(my_dlog, OPEN, item_type, item, &box);
			}

		ModalDialog(filter_list, &item_hit);
		switch(item_hit) {
			case NULL:
				/* these items don't do anything	*/
				/* such as list selections			*/
				break;
			case OPEN:
				/* first make SURE something is active */
				if(cell_active()) SysBeep(5);
				break;
			case QUIT:
				done = TRUE;
				break;
			case SCROLL:
				/* this event shouldn't happen since all */
				/* scroll/list events are intercepted	 */
				break;
			default:
				/* this is another unexpected event	*/
				break;
			}
		}

	DisposDialog(my_dlog);
	SetPort(window);
}
