/*
File SCSITypes.c, include file for SCSI.c and Driver.c
Leo Drizis, March 1989
Only for personal use, commercial use prohibited !!!

Contains all the typedefs for SCSI commands and parameters

ALL RESERVED FIELDS MUST BE SET TO ZERO !
*/

typedef unsigned char lsector[BLKSIZE];	/* one disk sector */

typedef struct fmtCmd {	/* format command */
	unsigned char opCode;	/* format opcode = 4 */
	unsigned char pars;	/* parameters */
	unsigned int rsrv1;
	unsigned char iLeave; /* interleave */
	unsigned char rsrv2;
	} fmtCmd;

typedef struct FmtData { /* format command data */
	unsigned char rsrv1;
	unsigned char FOV : 1; /* Format Options Valid */
	unsigned char rsrv2 : 1;
	unsigned char DCRT : 1; /* Disable CeRTification on format */
	unsigned char STPF : 1; /* SToP Format on error */
	unsigned char rsrv3 : 4;
	unsigned int DefLLen; /* defect list length */
	unsigned long DBA1,DBA2; /* two defect blocks */
	} FmtData;

typedef struct reWrCmd { /* read or write command */
	unsigned char opCode; /* opcode = 8(read) or 10(write) */
	unsigned char rsrv1;
	unsigned int addr; /* sector to start reading/writing */
	unsigned char length; /* no of sectors to read/write */
	unsigned char rsrv2;
	} reWrCmd;

typedef struct readDefCmd { /* read defects command */
	unsigned char opCode; /* opcode = 0x37 */
	unsigned char rsrv1;
	unsigned char which; /* user's or manufacturer's defects */
	unsigned char rsrv2[5];
	unsigned char allocLen; /* no of bytes to read */
	unsigned char rsrv3;
	} readDefCmd;

typedef struct modeSHead { /* mode sense header */
	unsigned char rsrv1;
	unsigned char mediumType; /* = 0 */
	unsigned char rsrv2;
	unsigned char BlDescLen; /* block descriptor length = 8 */
	} modeSHead;

typedef struct blockDesc { /* block descriptor */
	unsigned long density : 8; /* = 0 */
	unsigned long nBlocks : 24; /* no of sectors */
	unsigned long rsrv : 8;
	unsigned long blkLen : 24; /* length of each sector */
	} blockDesc;

typedef struct serNumPage { /* serial number page */
	unsigned char pageCode; /* = 0x20 */
	unsigned char pageLen; /* = 9 */
	unsigned char serNum[9]; /* the serial number */
	} serNumPage;

typedef struct fmtPage { /* format parameters page */
	unsigned char pageCode; /* = 3 */
	unsigned char pageLen; /* page length = 0x16 */
	unsigned int trPZone; /* tracks per cylinder */
	unsigned int altTrPZone; /* alternate tracks per cylinder */
	unsigned int rsrv;
	unsigned int altTrPVol; /* alternate tracks per volume */
	unsigned int secPTr; /* sectors per track */
	unsigned int bytPSect; /* bytes per sector */
	unsigned int iLeave; /* interleave */
	unsigned long rsrv2,rsrv3;
	} fmtPage;

typedef struct geoPage { /* drive geometry page */
	unsigned char pageCode; /* = 4 */
	unsigned char pageLen; /* page length = 0x12 */
	unsigned long nCyl : 24; /* number of cylinders */
	unsigned long nHead : 8; /* number of heads */
	unsigned char rsrv[14];
	} geoPage;

typedef struct modeSenseCmd { /* mode sense command */
	unsigned char opCode; /* = 0x15 */
	unsigned char LUN : 3; /* Logical Unit Number = 0 */
	unsigned char rsrv1 : 5;
	unsigned char PCF : 2; /* Page Control Field */
	unsigned char pageCode : 6; /* requested page code */
	unsigned char rsrv2;
	unsigned char allocLen; /* no of bytes to read */
	unsigned char rsrv3;
	} modeSenseCmd;

typedef struct inqData { /* inquiry parameters */
	unsigned char devTypCod; /* device type code = 0 */
	unsigned char devTypQual; /* device type qualifier */
	unsigned char revis; /* revision level */
	unsigned char rsrv;
	unsigned char addLen; /* no of additional bytes to read */
	unsigned char rsrv2[3];
	char manuf[8]; /* manufacturer's name */
	char prod[16]; /* product name */
	unsigned char hardRev; /* hardware revision level */
	unsigned char firmRev; /* firmware revision level */
	unsigned char ROMRev; /* ROM revision level */
	unsigned char rsrv3;
	char serNum[9]; /* serial number */
	} inqData;

typedef struct defHead { /* defect list header */
	unsigned char rsrv;
	unsigned char which; /* user's or manufacturer's defects */
	unsigned int len; /* length of defect list to follow */
	} defHead;

typedef struct senseData { /* request sense data */
	unsigned char valid : 1; /* if data is valid */
	unsigned char errClass : 3; /* error class */
	unsigned char errCode : 4; /* error code */
	unsigned char rsrv1;
	unsigned char rsrv2 : 4;
	unsigned char senseKey : 4; /* sense key */
	unsigned char LBA_MSB;
	unsigned long LBA_LSB : 24; /* sector no */
	unsigned long addSenLen : 8; /* additional sense length */
	unsigned char rsrv3[4];
	unsigned char addSenCode; /* additional sense code */
	unsigned char rsrv4[4];
	unsigned char retries; /* retries needed */
	unsigned int  pCyl; /* physical cylinder */
	unsigned char pHead; /* physical head */
	unsigned char pSect; /* physical sector */
	} senseData;

typedef struct scsiCmd { /* general SCSI command */
	unsigned char	opCode;
	unsigned char	LUN; /* Logical Unit Number = 0 */
	unsigned int	LBA; /* sector number */
	unsigned char	lng; /* length */
	unsigned char	rsrv;
	} scsiCmd;

typedef struct tib { /* Transfer Instruction Block */
	unsigned int	cmd; /* command */
	Ptr		add; /* address of read/write buffer */
	long	cnt; /* number of bytes to read/write */
	} tib;

typedef struct ReassBlkLst { /* reassign blocks list */
	unsigned char	rsrv1;
	unsigned char	rsrv2;
	unsigned int	leng; /* length of list */
	unsigned long	defLBA; /* defect sector */
	} ReassBlkLst;

typedef struct senseBytes { /* short form of sense bytes */
	unsigned char errCode;
	unsigned char rsrv;
	unsigned char senseKey;
	unsigned char filler;
	unsigned long LBA : 24;
	unsigned long addLng : 8;
	} senseBytes;

typedef struct myStruct { /* partition status data */
	DrvSts2	myStats; /* drive status variables */
	unsigned long	dataStart; /* starting sector on disk */
	unsigned long	dataCnt; /* number of sectors in partition */
	} myStruct;


